# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .agent_message_guideline_rationale_dto import AgentMessageGuidelineRationaleDto
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class AgentMessageGuideline(UniversalBaseModel):
    action: str = pydantic.Field()
    """
    A single action that explains what to say; i.e. "Tell the customer that you are thinking and will be right back with an answer."
    """

    rationale: typing.Optional[AgentMessageGuidelineRationaleDto] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow", frozen=True
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
