# This file was auto-generated by Fern from our API Definition.

import typing

CompositionModeDto = typing.Union[
    typing.Literal["fluid", "canned_fluid", "composited_canned", "strict_canned"],
    typing.Any,
]
