# This file was auto-generated by Fern from our API Definition.

import typing

AgentMessageGuidelineRationaleDto = typing.Union[
    typing.Literal["unspecified", "buy_time", "follow_up"], typing.Any
]
