# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.session_listing import SessionListing
from ...types.session import Session

SessionsListResponse = typing.Union[SessionListing, typing.List[Session]]
