# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.paginated_customers import PaginatedCustomers
from ...types.customer import Customer

CustomersListResponse = typing.Union[PaginatedCustomers, typing.List[Customer]]
